      DOUBLE PRECISION FUNCTION ROZTST(NQ,A)
C  This routine conducts Rozanov's spectral teset for the invertibility
C  of a univarite lag operator.  If the operator is invertible then the
C  result should be zero, otherwise it is positive.  A good definition
C  of algorithmically zero with this routine is 10^(-6).  This can be made
C  a little smaller, about 10^(-7), with stricter tolerances in the
C  quadrature routine, but at the expense of a 40% increase in computing time.
C  Inputs:
C  A       A(j) contains the term on the j'th lag.  The contemnporaneous term
C          in A(0) must be 1.0.
C  NQ      Length of lag operator
C  Output:
C  ROZTST  Rozanov's test criterion
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      EXTERNAL QFUNC
      DIMENSION A(0:NQ)
      DATA PI/3.1415926535D0/
      COMMON/QQ1/AA(0:200),NNQ
      PI=DCONST('PI')
      NNQ=NQ
      CALL DCOPY(NQ+1,A,1,AA,1)
      CALL DQDAG(QFUNC,0.0D0,PI,1.0D-6,1.0D-6,2,HOLD,ERR)
      ROZTST=HOLD/PI
      RETURN
      END
      DOUBLE PRECISION FUNCTION QFUNC(ALAM)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z) 
      COMMON/QQ1/A(0:200),NQ
      AA=0.0D0
      BB=0.0D0
      DO 10 IQ=0,NQ
      ARG=ALAM*IQ
      AA=AA+A(IQ)*DCOS(ARG)
   10 BB=BB+A(IQ)*DSIN(ARG)
      QFUNC=DLOG(AA**2+BB**2)
      RETURN
      END
