      SUBROUTINE TRMARK(LFWD,N,VP,VPTR,TRJAC)
C  In the forward transformation --
C  This routine performs a normalizing transformation for a Markov transition
C  matrix.  The tranformation is from pij to qij=log(pij/pii) for all j.ne.i.
C  It also returns the log Jacobian of transformation.  This is what you need
C  to add to the log density in the new parameterization.
C  Inputs:
C  LFWD   Set .TRUE.
C  N      Order of Markov matrix
C  VP     Markov matrix, stored in vectorized form (by columns)
C  Outputs:
C  VPTR   Transformed parameters, organized by **rows**
C  TRJAC  Log Jacobian of forward transformation
C
C  In the backward transformation --
C  Inverse transformation is performed; TRJAC is negative of forward value
C  Inputs:
C  LFWD   Set .FALSE.
C  N      Order of Markov matrix
C  VPTR   Transformed parameters (qij), stored in vectorized form (by rows)
C  Outputs:
C  VP     Markov matrix, stored in vectorized form (by columns)
C  TRJAC  Log Jacobian of backward transformation
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      LOGICAL LFWD
      PARAMETER (LD=100)
      COMMON/SCRATCH/V1(LD),V2(LD),A1(LD,LD),A2(LD,LD)
      DIMENSION VP(N*N),VPTR(N*(N-1))
      IF(.NOT.LFWD)GO TO 100
      CALL UNVEC(N,N,VP,A1,LD)
      AA=0.0D0
      L=0
      DO 20 I=1,N
      BB=DLOG(A1(I,I))
      DO 10 J=1,N
      IF(J.EQ.I)GO TO 10
      L=L+1
      CC=DLOG(A1(I,J))
      VPTR(L)=CC-BB
      AA=AA+CC
   10 CONTINUE
   20 AA=AA+BB
      TRJAC=AA
      RETURN
  100 L=0
      AA=0.0D0
      DO 120 I=1,N
      BB=0.0D0
      DO 110 J=1,N
      A1(I,J)=1.0D0
      IF(I.EQ.J)GO TO 110
      L=L+1
      AA=AA-VPTR(L)
      A1(I,J)=DEXP(VPTR(L))
  110 BB=BB+A1(I,J)
      AA=AA+N*DLOG(BB)
      DO 120 J=1,N
  120 A1(I,J)=A1(I,J)/BB
      CALL VEC(N,N,A1,LD,VP)
      TRJAC=AA
      RETURN
      END
