      SUBROUTINE TRPD(LFWD,N,VA,VATR,TRJAC)
C  In the forward transformation --
C  This routine performs a normalizing transformation on a symmetric positive
C  definite matrix, and returns the log Jacobian of transformation.
C  The transformation is based on the Choleski factorization of p.d. A,
C  A = LL', where L is lower triangular.  The transformation is to the logs
C  of the diagonal elemnts of L, and the untransformed elements in the strict
C  lower triangle of A.  The log Jacobian is what you add to the log density
C  of the original distribution, to get the right log density for the
C  transformed variables.
C  Inputs:
C  LFWD    Set .TRUE.
C  N       Order of p.d. matrix
C  VA      p.d. matrix in lower triangular storage, by columns
C  Outputs:
C  VATR    Transformed Choleski decomposition, same storage convention as VA
C  TRJAC   Log Jacobian of forward transformation
C
C  In the backward transformation --
C  Inverse transformation is performed; TRJAC is negative of forward value
C  Inputs:
C  LFWD    Set .FALSE.
C  N       Order of p.d. matrix
C  VATR    Transformed Choleski decomposition of  matrix, lower tri. by columns
C  Outputs:
C  VA      p.d. matrix in lower triangular storage, by columns
C  TRJAC   Log Jacobian of backward transformation
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DATA DLOG2/.693147181D0/
      LOGICAL LFWD
      DIMENSION VA(N*(N+1)/2),VATR(N*(N+1)/2)
      PARAMETER(LD=100)
      COMMON/SCRATCH/V1(LD),V2(LD),A1(LD,LD),A2(LD,LD)
      IF(.NOT.LFWD)GO TO 100
      L=1
      DO 10 J=1,N
      DO 10 I=J,N
      A1(J,I)=VA(L)
   10 L=L+1
      CALL DLFTDS(N,A1,LD,A2,LD)
      AA=N*DLOG2
      K=N+1
      L=0
      IF(N.EQ.1)GO TO 40
      DO 30 J=1,N-1
      L=L+1
      BB=DLOG(A2(J,J))
      VATR(L)=BB
      AA=AA+K*BB
      DO 20 I=J+1,N
      L=L+1
   20 VATR(L)=A2(J,I)
   30 K=K-1
   40 BB=DLOG(A2(N,N))
      VATR(L+1)=BB
      TRJAC=AA+K*BB
      RETURN
  100 L=0
      K=N+1
      AA=-N*DLOG2
      DO 120 J=1,N
      DO 110 I=J,N
      L=L+1
  110 A1(I,J)=VATR(L)
      AA=AA-K*A1(J,J)
      K=K-1
  120 A1(J,J)=DEXP(A1(J,J))
      TRJAC=AA
      L=0
      DO 140 J=1,N
      DO 140 I=J,N
      L=L+1
      AA=0.0D0
      DO 130 M=1,J
  130 AA=AA+A1(I,M)*A1(J,M)
  140 VA(L)=AA
      RETURN
      END
