      DOUBLE PRECISION FUNCTION YLC(G,G1,G2,D1,D2,LXHAT,XHAT)
C  This function draws from a distribution whose log density is strictly
C  concave.  Documentation is in notes on the ARMA(p,q) paper.
C  Inputs:
C  G       Function G(X) that has input ordinate X and returns log of density
C          kernel, G
C  G1      Function G1(X) that has input ordinate X and returns first
C          derivative of log of density kernel, G1
C  G2      Function G2(X) that has input ordinate X and returns second
C          derivative of log of density kernel, G2
C  D1      Lower end of finite support
C  D2      Upper end of finite support
C  LXHAT   .TRUE. if mode of log density kernel provided on input
C  Input/output:
C  XHAT    Mode of log density kernel
C  Output:
C  YLC     Draw from distribution
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      PARAMETER(LROW=500)
      DIMENSION T(LROW,5)
c     ,V1(2),V2(2)
C     DIMENSION TEMP(5)
      EXTERNAL G,G1,G2
      LOGICAL LXHAT
      C1=D1
      C2=D2
      IF(C2.LT.C1)GO TO 1050
      IF(C2.GT.C1)GO TO 10
      YLC=C1
      RETURN 
   10 CALL DSET(5*LROW,0.0D0,T,1)
C  Step 1
      G1C1=G1(C1)
      IF(G1C1.GE.0.0D0)GO TO 110
      X1=UMIN(C1-1.0D0/G1C1,C1+.5D0*(C2-C1))
      X2=UMIN(X1-1.0D0/G1(X1),C1+.9D0*(C2-C1))
      GO TO 150
  110 G1C2=G1(C2)
      IF(G1C2.LE.0.0D0)GO TO 120
      X2=UMAX(C2-1.0D0/G1C2,C1+.5D0*(C2-C1))
      X1=UMAX(X2-1.0D0/G1(X2),C1+.1D0*(C2-C1))
      GO TO 150
  120 IF(LXHAT)GO TO 130
      MAXFN=1000
      AA=C1
      XHAT=C2
      CALL DZBREN(G1,0.1D0,0.0D0,AA,XHAT,MAXFN)
  130 S=1.0D0/DSQRT(-G2(XHAT))
      X1=DMAX1(XHAT-S,C1+.5D0*(XHAT-C1),C1+.1D0*(C2-C1))
      X2=DMIN1(XHAT+S,XHAT+.5D0*(C2-XHAT),C2+.1D0*(C1-C2))
C  Robustify to scaling problems so that subsequent exponentiation will not
C  overflow or underflow.
  150 XBAR=0.5D0*(X1+X2)
      SCLOG=-G(XBAR)
      SC=DSQRT(-G2(XBAR))
      X1=SC*X1
      X2=SC*X2
      C1=SC*C1
      C2=SC*C2
      SC1=1.0D0/SC
C  Step 2
  200 T(1,1)=X1
      T(2,1)=X2
      T(1,3)=G(SC1*X1)+SCLOG
      T(2,3)=G(SC1*X2)+SCLOG
      T(1,4)=G1(SC1*X1)*SC1
      T(2,4)=G1(SC1*X2)*SC1
      T(1,2)=C1
      T(2,2)=(T(2,3)-T(1,3)+T(1,4)*T(1,1)-T(2,4)*T(2,1))
     1                                      /(T(1,4)-T(2,4))
      T(3,2)=C2
      T(1,5)=(DEXP(T(1,3)+T(1,4)*(T(2,2)-T(1,1)))
     1       -DEXP(T(1,3)+T(1,4)*(T(1,2)-T(1,1))))/T(1,4)
      T(2,5)=T(1,5)+(DEXP(T(2,3)+T(2,4)*(T(3,2)-T(2,1)))
     1              -DEXP(T(2,3)+T(2,4)*(T(2,2)-T(2,1))))/T(2,4)
C  Step 3
  300 JCAP=2
C  Step 4
  400 TSTAR=T(JCAP,5)*DRNUNF()
C  Step 5
  500 DO 510 I=1,JCAP
      IF(TSTAR.GT.T(I,5))GO TO 510
      J=I
      GO TO 600
  510 CONTINUE
      GO TO 1010
C  Step 6
  600 ZJ=0.0D0
      IF(J.GT.1)ZJ=T(J-1,5)
C     V1(1)=T(J,4)*(TSTAR-ZJ)
C     V1(2)=1.0D0
C     V2(1)=-T(J,3)
C     V2(2)=T(J,4)*(T(J,2)-T(J,1))
C     X=T(J,1)+DLGLCE(2,V1,V2)/T(J,4)
      X=T(J,1)  +  (DLOG(T(J,4)*(TSTAR-ZJ)*DEXP(-T(J,3))
     1              +DEXP(T(J,4)*(T(J,2)-T(J,1)))))        /T(J,4)  
C  Step 7
  700 S3=G(SC1*X)+SCLOG
      IF(DRNUNF().GE.DEXP(S3-T(J,3)-T(J,4)*(X-T(J,1))))GO TO 800
C     CALL DOUT(SC1,1,'SC1')
C     CALL DOUT(X,1,'X')
      ZZZ=T(J,4)*(TSTAR-ZJ)*DEXP(-T(J,3))
     1              +DEXP(T(J,4)*(T(J,2)-T(J,1)))
C     CALL DOUT(ZZZ,1,'ZZZ')
      CALL DCOPY(5,T(J,1),LROW,TEMP,1)
C     CALL DOUT(TEMP,5,'T line')
C     CALL DOUT(TSTAR,1,'TSTAR')
C     CALL DWRRRN('T',JCAP,5,T,LROW,0)
      YLC=SC1*X
      RETURN
C  Step 8
  800 JT=J
      IF(X.GT.T(J,1))JT=J+1
      JTM=JT-1
      JTP=JT+1
C  Step 8(a)
      JCAP=JCAP+1
      IF(JCAP.EQ.LROW)GO TO 1030
      DO 810 J=JCAP,JT,-1
      J1=J+1
      DO 810 K=1,5
  810 T(J1,K)=T(J,K)
C  Step 8(b)
      T(JT,1)=X
      T(JT,3)=S3
      T(JT,4)=G1(SC1*X)*SC1
C  Step 8(c)
      IF(JT.GT.1)T(JT,2)
     1  =(T(JT,3)-T(JTM,3)+T(JTM,4)*T(JTM,1)-T(JT,4)*T(JT,1))
     2                                           /(T(JTM,4)-T(JT,4))
      IF(JT.LT.JCAP)T(JTP,2)
     1  =(T(JTP,3)-T(JT,3)+T(JT,4)*T(JT,1)-T(JTP,4)*T(JTP,1))
     2                                           /(T(JT,4)-T(JTP,4))
C  Step 8(d)
      IF(JT.GT.1)YA=(DEXP(T(JTM,3)+T(JTM,4)*(T(JT,2)-T(JTM,1)))
     2     -DEXP(T(JTM,3)+T(JTM,4)*(T(JTM,2)-T(JTM,1))))/T(JTM,4)
      YB=(DEXP(T(JT,3)+T(JT,4)*(T(JTP,2)-T(JT,1)))
     2     -DEXP(T(JT,3)+T(JT,4)*(T(JT,2)-T(JT,1))))/T(JT,4)
      IF(JT.LT.JCAP)YC=(DEXP(T(JTP,3)+T(JTP,4)*(T(JT+2,2)-T(JTP,1)))
     2     -DEXP(T(JTP,3)+T(JTP,4)*(T(JTP,2)-T(JTP,1))))/T(JTP,4)     
C  Step 8(e)
      IF(JT.EQ.1)GO TO 820
      A1=0.0D0
      IF(JT.GE.3)A1=T(JT-2,5)
      T(JTM,5)=A1+YA
C  Step 8(f)
  820 A1=0.0D0
      IF(JT.GE.2)A1=T(JTM,5)
      T(JT,5)=A1+YB
C  Step 8(g)
      IF(JT.EQ.JCAP)GO TO 400
      DEL=T(JT,5)+YC-T(JTP,5)
      DO 830 J=JTP,JCAP
  830 T(J,5)=T(J,5)+DEL
C  Step 9
      GO TO 400
C  Errors
 1010 WRITE(6,1020)
 1020 FORMAT(/,'  Adding-up violations in YLC.')
      CALL DWRRRN('T',JCAP+1,5,T,LROW,0)
      CALL DOUT(X1,1,'X1        ')
      CALL DOUT(X2,1,'X2        ')
      CALL TERM
 1030 CALL PEND('  Storage capacity in YLC exhausted')
 1050 WRITE(6,1060)C1,C2
 1060 FORMAT(/,' Constraint interval [',1PE15.5,',',1PE15.5,'] in YLC')
      CALL TERM
      END
