#include "pset.h"


/*
 * gen_powerset(pset): generates a binary representation
 * of MAXSIZE powersets of size i for 0 < i <= MAXSIZE, 
 * stores in pset
 */
void gen_powerset(int ***powerset, int MAXSIZE)
{
  int num_subsets, i, j, k, pulse;
  int is_member[MAXSIZE];

  for (i = 0; i < MAXSIZE; i++) { is_member[i] = 1; }

  for (i = 0; i < MAXSIZE; i++)
    {
      num_subsets = pow(2, i);

      for (j = 0; j < num_subsets; j++)
	{
	  for (k = 0; k < i; k++)
	    {
	      /* determine pulse size for set member */
	      pulse = pow(2, k);
	      
	      /* if pulse is reached, flip membership bit */
	      if (j % pulse == 0)
		{ is_member[k] = (is_member[k] + 1) % 2; }
	      
	      powerset[i][j][k] = is_member[k];
/*
if(i==4)
  printf("%d %d %d %d\n",i,j,k,powerset[i][j][k]);
*/
	    }
	}
    }
}

/*
 * alloc_powerset(pset): allocates minimal space in pset for binary 
 * representation of MAXSIZE powersets of size i for 0 < i <= MAXSIZE
 */
void alloc_powerset(int ***powerset, int MAXSIZE)
{
  int num_subsets, i, j;
  
  for(i = 0; i < MAXSIZE; i++)
    {
      num_subsets = pow(2, i);
      
      powerset[i] = (int **) malloc(num_subsets * sizeof(int *));
      
      for (j = 0; j < num_subsets; j++)
	{
	  powerset[i][j] = (int *) malloc(i * sizeof(int)); 
	}
    }  
}

/*
 * free_powerset(pset): frees allocated space in pset
 */
void free_powerset(int ***powerset, int MAXSIZE)
{
  int num_subsets, i, j;
  
  for(i = 0; i < MAXSIZE; i++)
    {
      num_subsets = pow(2, i);
      
      for (j = 0; j < num_subsets; j++)
	{
	  free(powerset[i][j]);
	}
      
      free(powerset[i]); 
    }  
}

/*Random Number generator*/

float ran1(long *idum)
{
        int j;
        long k;
        static long iy=0;
        static long iv[NTAB];
        float temp;
 
        if (*idum <= 0 || !iy) {
                if (-(*idum) < 1) *idum=1;
                else *idum = -(*idum);
                for (j=NTAB+7;j>=0;j--) {
                        k=(*idum)/IQ;
                        *idum=IA*(*idum-k*IQ)-IR*k;
                        if (*idum < 0) *idum += IM;
                        if (j < NTAB) iv[j] = *idum;
                }
                iy=iv[0];
        }
        k=(*idum)/IQ;
        *idum=IA*(*idum-k*IQ)-IR*k;
        if (*idum < 0) *idum += IM;
        j=iy/NDIV;
        iy=iv[j];
        iv[j] = *idum;
        if ((temp=AM*iy) > RNMX) return RNMX;
        else return temp;
}

double error_check(double gamma)
{
  long seed, dummy;
  float dice = 0.0;
  time_t current_time;
  int count;

  time( &current_time);
  srand(current_time);
 
  dummy= -rand();

  seed =  -rand();
  dice = ran1(&seed);

  if(dice<0.0001){
    seed = -rand();
    dice = ran1(&seed);
    gamma = gamma + ((0.5 - dice) * EPS);
  }

  return gamma;
 
}
