#include "struct.h"
#include "paramE.h"
#include "vars.h"

double logDelay(compVars var, int nAtt)
{
  return (nAtt - 1)*var.gamma*log((var.yStar-var.yLower) / 
                        (var.yBar-var.yLower));
}

double calProbAgree(compVars var, int dur, int maxDur)
{
  double probAgree(double y)
  {
     double aa=(y-var.yLower)/(var.yBar-var.yLower);
     double pa=(dur-LB)/(maxDur-LB);
     double a=var.delta*(y-LB)/(maxDur-y), c=var.delta+a;
     pa = pow(pa, a-1) * pow(1-pa, var.delta-1)/(maxDur-LB);
     pa *= exp(DLNGAM(&c)-DLNGAM(&a)-DLNGAM(&(var.delta)));
     pa *= var.gamma * pow(aa, var.gamma - 1) / (var.yBar-var.yLower);
     return pa;
   }
   qgaus(probAgree, var.yStar, var.yBar);
}

double calResCake(double beta, compVars var)
{
  double resCake(double y)
  {
       double gamma1=var.gamma+1;
       double res;

       res=var.gamma*(var.yBar-var.yLower);
       res+=pow(y-var.yLower,gamma1)/pow(var.yBar-var.yLower,var.gamma);
       res=res/gamma1+var.yLower;
       return y-beta*res;
  }
  fixedpoint(resCake, var.yLower, beta*var.yBar);
}


double calPayoff(double param[], compVars var)
{
  return var.yStar*(1-param[BETA]*(1-var.pi))/param[BETA];
}

void possibleCoalitions(double param[], dataVars obs, 
                        double payoffs[maxNumCoal])
{
  int i,j;
  dataVars posObs;
  compVars var;

  copyObsExog(obs,&posObs);
  for(i = 0; i < pow(2, posObs.numP-1); i++)
    {
      posObs.inout[posObs.proposer[0]]=1;
      for(j = 0; j < posObs.numP; j++)
	{
          if(j< posObs.proposer[0])
            posObs.inout[j]=powerset[posObs.numP-1][i][j]; 
          if(j> posObs.proposer[0])
            posObs.inout[j]=powerset[posObs.numP-1][i][j-1]; 
	}
      setAll(&posObs); 
      var.yBar=calYBar(param, posObs);
      posObs.yBar=var.yBar;
      var.yLower=calYLower(param, posObs);
      var.gamma=calGamma(param, posObs);
      var.yStar=calResCake(param[BETA], var);
      var.pi=calPi(param,posObs);
      payoffs[i] = calPayoff(param, var);
    }
}

void logLikelihood(long int *numParam, double parameters[], double *logLike)
{
  double payoffs[maxNumCoal]={0};
  double param[*numParam], probAgree; 
  compVars var;
  double formPayoff;
  int i, k;

  DCOPY(numParam,parameters, &one, param,&one);
  if (transformed) {
     param[RHO]=exp(param[RHO]);
     param[BETA]=exp(param[BETA]);
     param[BETA]=param[BETA]/(param[BETA]+1);
  }
  if (transformed)
     for (i=0; i < *numParam; i++)
        printf("%d %7.6f \n",i,param[i]);
  fflush(stdout);
  *logLike = 0.0;
  for(i = 0; i < nObs; i++) {
     var.yBar = calYBar(param,data[i]);
     data[i].yBar=var.yBar;
     var.yLower = calYLower(param,data[i]);
     var.gamma = calGamma(param, data[i]);
  if (transformed)
  printf("%d %d %5.4f\n",data[i].cNum,data[i].govt, var.gamma);
     var.delta = calDelta(param, data[i]);
     var.yStar = calResCake(param[BETA], var);
     probAgree = calProbAgree(var, data[i].dur,data[i].maxDur);
     *logLike += log(probAgree);
     *logLike += logDelay(var, data[i].nAtt);	 
     *logLike += logFormateur(param, data[i]);
     possibleCoalitions(param, data[i], payoffs);
     var.pi=calPi(param,data[i]);
     formPayoff=calPayoff(param, var);
     *logLike += logProbCoalition(formPayoff, payoffs, param, data[i]);
     *logLike=*logLike-(data[i].nAtt-1)*log(calPiDenom(param,data[i]));
        for(k=1; k<data[i].nAtt; k++)
           *logLike += calLogPiNum(param,data[i],data[i].proposer[k]);
     DSET(&nm,&zero,payoffs,&one);
   }
  if (transformed) {
     printf("log likelihood: %lf \n",*logLike);
     fflush(stdout);
  }
  *logLike=  -*logLike;
}
