/* pset.h */
#ifndef __PSET_H
#define __PSET_H

/* System includes */
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>

/*Preporcessor Defines*/
/* note #undef's at end of file */
#define IA 16807
#define IM 2147483647
#define AM (1.0/IM)
#define IQ 127773
#define IR 2836
#define NTAB 32
#define NDIV (1+(IM-1)/NTAB)
#define EPS 1.2e-7
#define RNMX (1.0-EPS)


/* Function prototypes */
/*
 * gen_powerset(pset, size): generates a binary representation
 * of 'size' powersets of size i for 0 < i <= 'size', 
 * stores in pset
 */

void gen_powerset(int ***, int);

/*
 * alloc_powerset(pset, size): allocates minimal space in pset for binary 
 * representation of 'size' powersets of size i for 0 < i <= 'size'
 */
void alloc_powerset(int ***, int);

/*
 * free_powerset(pset, size): frees allocated space in pset
 */
void free_powerset(int ***, int);

double error_check(double gamma);

#endif __PSET_H


